/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.3 Automatisches Lernen
 * Ganglinien
 * Copyright (C) 2019 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllernen.gllernen.portale;

import java.time.Duration;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.iav.gllernen.gllernen.GlLernen;
import de.bsvrz.iav.gllernen.gllernen.domain.GanglinienLernenParameterLeser;
import de.bsvrz.iav.gllernen.gllernen.domain.GanglinienModellAutomatischesLernen;
import de.bsvrz.iav.gllib.gllib.shared.CronPattern;
import de.bsvrz.iav.gllib.gllib.shared.Event;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementierung eines {@link GanglinienLernenParameterLeser} mit der Kopplung
 * zu einem Datenverteiler-System.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavGanglinienLernenParameterLeser implements GanglinienLernenParameterLeser {

    private static final Debug LOGGER = Debug.getLogger();

    private final ClientReceiverInterface receiver = new ClientReceiverInterface() {

        @Override
        public void update(ResultData[] results) {
            for (ResultData result : results) {
                parameterGeaendertEvent.send(konvertiere(result));
            }
        }
    };

    private final Event<GanglinienModellAutomatischesLernen> parameterGeaendertEvent = new Event<>();

    private AttributeGroup atg;

    private Aspect asp;

    private final GlLernen applikation;

    private ClientDavInterface dav;

    public DavGanglinienLernenParameterLeser(GlLernen applikation, ClientDavInterface dav) {
        this.applikation = applikation;
        this.dav = dav;
        atg = dav.getDataModel().getAttributeGroup("atg.ganglinienModellAutomatischesLernen");
        asp = dav.getDataModel().getAspect("asp.parameterSoll");
        dav.subscribeReceiver(receiver, dav.getLocalConfigurationAuthority(), new DataDescription(atg, asp),
                ReceiveOptions.normal(), ReceiverRole.receiver());
    }

    @Override
    public Event<GanglinienModellAutomatischesLernen> parameterGeaendert() {
        return parameterGeaendertEvent;
    }

    private GanglinienModellAutomatischesLernen konvertiere(ResultData result) {

        if (result.hasData()) {
            String intervall = result.getData().getTextValue("AlgAktualisierungsintervall").getText();
            Duration duration = Duration.ofDays(result.getData().getUnscaledValue("AlgDatenMindestalter").longValue());
            int abstand = result.getData().getUnscaledValue("AlgMaxVergleichsAbstand").intValue();

            try {
                return GanglinienModellAutomatischesLernen.of(CronPattern.parse(intervall), abstand, duration);
            } catch (IllegalArgumentException e) {
                LOGGER.warning(e.getLocalizedMessage());
                applikation.warnung(e.getLocalizedMessage());
            }
        }

        return GanglinienModellAutomatischesLernen.STANDARD;
    }

    @Override
    public GanglinienModellAutomatischesLernen getParameter() {
        return konvertiere(dav.getData(dav.getLocalConfigurationAuthority(), new DataDescription(atg, asp), 0));
    }
}
